package com.elab.mq.listener;

import com.elab.mq.api.Action;
import com.elab.mq.api.ConsumeContext;
import com.elab.mq.model.MessageModel;
import com.elab.mq.msg.IConsumeProcessService;
import com.elab.mq.msg.adptor.MessageListenerAdaptor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

/**
 * 订阅消息监听
 * <p>
 * 默认的消费者
 */
public class DefaultMessageListener extends MessageListenerAdaptor {

    private Logger logger = LoggerFactory.getLogger(getClass());

    private ConcurrentMap<String, IConsumeProcessService> tagMap = new ConcurrentHashMap<String, IConsumeProcessService>();

    private List<IConsumeProcessService> consumeProcessServiceList;

    public void setConsumeProcessServiceList(List<IConsumeProcessService> consumeProcessServiceList) {
        this.consumeProcessServiceList = consumeProcessServiceList;
    }

    public void init() {
        if (this.consumeProcessServiceList.size() > 0) {
            for (int i = 0; i < consumeProcessServiceList.size(); i++) {
                IConsumeProcessService consumeProcessService = consumeProcessServiceList.get(i);
                String tag = consumeProcessService.tag();
                if (tag == null) {
                    logger.error(" 消息监听tag为Null异常 : " + consumeProcessService.toString() + " tag方法返回为null !!!");
                    continue;
                }
                IConsumeProcessService tagService = tagMap.get(tag);
                if (tagService != null) {
                    logger.error(" 消息监听tag重复异常 : " + consumeProcessService.toString() + " tag方法匹配有重复 !!! 重复类 :  " + tagService.toString());
                    continue;
                }
                logger.debug(" 注册消费监听器 : key - " + tag + " value - " + consumeProcessService.toString());
                tagMap.putIfAbsent(tag, consumeProcessService);
            }
        }
    }

    @Override
    public Action consume(MessageModel messageModel, ConsumeContext consumeContext) {
        String tag = messageModel.getTag();
        try {
            logger.debug(" 获取到的消息对象 : " + messageModel.toString());
            Thread.sleep(3000);
            IConsumeProcessService consumeProcessService = tagMap.get(tag);
            if (consumeProcessService == null) {
                logger.error(" 消息消费失败 ... 找不到对应的消费监听处理执行类 关注Tag = " + tag);
                return Action.ReconsumeLater;
            }

            boolean process = consumeProcessService.process(messageModel, consumeContext);

            if (!process) {
                logger.error(" 消息消费失败 : 消息内容 " + messageModel.toString() + " 等待重试..");
                return Action.ReconsumeLater;
            }
            logger.debug(" 消息消费成功 :  " + messageModel.getMsgID());
            return Action.CommitMessage;
        } catch (Exception e) {
            logger.error(" 事务消息执行重试操作 ... " + e.getMessage(), e);
            //消费失败
            return Action.ReconsumeLater;
        }
    }


}