package com.elab.mq.msg.impl;

import com.aliyun.openservices.ons.api.SendResult;
import com.aliyun.openservices.ons.api.bean.ProducerBean;
import com.elab.mq.model.MessageModel;
import com.elab.mq.model.SendResultModel;
import com.elab.mq.msg.IMsgProducerFacade;
import com.elab.mq.msg.adptor.SendCallbackAdaptor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Date;

/**
 * 消息生产包装类
 *
 * @author Liukx
 * @create 2018-04-09 18:23
 * @email liukx@elab-plus.com
 **/
public class MsgProducerImpl extends ProducerBean implements IMsgProducerFacade {

    private Logger logger = LoggerFactory.getLogger(MsgProducerImpl.class);

    @Override
    public void sendAsync(MessageModel message, SendCallbackAdaptor sendCallback) {
        logger.debug(" 发送一条异步消息 " + message + " -> " + message.toString() + " 回调类 : " + sendCallback.getClass().getName());
        super.sendAsync(message, sendCallback);
    }

    @Override
    public SendResultModel send(MessageModel message) {
        logger.debug(" 发送一条消息 " + message.getMsgID() + " -> " + message.toString());
        SendResult result = super.send(message);
        logger.debug(" 消息发送结果 : " + result.toString());
        return new SendResultModel(result);
    }

    @Override
    public void sendOneway(MessageModel message) {
        logger.debug(" 发送一条单向消息 " + message.getMsgID() + " -> " + message.toString());
        super.sendOneway(message);
    }

    @Override
    public SendResultModel sendDeliverTime(MessageModel messageModel, int seconds) {
        logger.debug(" 发送一条延时消息 : 将在[" + seconds + "]秒后发送 , 消息内容 : " + messageModel.toString());
        long time = System.currentTimeMillis() + (seconds * 1000);
        messageModel.setStartDeliverTime(time);
        return send(messageModel);
    }

    @Override
    public SendResultModel sendTimingMsg(MessageModel messageModel, Date datetime) {
        logger.debug(" 发送一条定时消息 : 将在[" + datetime + "]后发送 , 消息内容 : " + messageModel.toString());
        messageModel.setStartDeliverTime(datetime.getTime());
        return send(messageModel);
    }
}
